@@ -1,4 +1,3 @@
-
 /*
  * (C) Copyright 2005- ECMWF.
  *
@@ -108,6 +107,9 @@ static int extra_set(grib_accessor* a, long val)
     grib_get_long(hand, "is_aerosol", &is_aerosol);
     grib_get_long(hand, "is_aerosol_optical", &is_aerosol_optical);
 
+    const int is_wave        = grib_is_defined(hand, "waveDirectionNumber");
+    const int is_wave_prange = grib_is_defined(hand, "typeOfWavePeriodInterval");
+
     switch (self->index) {
         case 0:
             /* class */
@@ -287,6 +289,11 @@ static int extra_set(grib_accessor* a, long val)
             return GRIB_INTERNAL_ERROR;
     }
 
+    if (is_wave || is_wave_prange) {
+        // ECC-1867
+        productDefinitionTemplateNumberNew = -1;  // disable PDT selection
+    }
+
     if (productDefinitionTemplateNumberNew >= 0) {
         grib_get_long(hand, self->productDefinitionTemplateNumber, &productDefinitionTemplateNumber);
         if (productDefinitionTemplateNumber != productDefinitionTemplateNumberNew)
