@@ -507,6 +507,7 @@ grib_handle* grib_handle_new_from_samples(grib_context* c, const char* sample_na
  * @return            the new handle, NULL if the message is invalid or a problem is encountered
  */
 grib_handle* grib_handle_clone(const grib_handle* h);
+grib_handle* grib_handle_clone_headers_only(const grib_handle* h);
 
 /**
  *  Frees a handle, also frees the message if it is not a user message
@@ -732,7 +733,7 @@ int grib_nearest_find_multiple(const grib_handle* h, int is_lsm,
 /*! \defgroup get_set Accessing header and data values   */
 /*! @{ */
 /**
- *  Get the number offset of a key, in a message if several keys of the same name
+ *  Get the byte offset of a key. If several keys of the same name
  *  are present, the offset of the last one is returned
  *
  * @param h           : the handle to get the offset from
@@ -745,7 +746,7 @@ int grib_get_offset(const grib_handle* h, const char* key, size_t* offset);
 /**
  *  Get the number of coded value from a key, if several keys of the same name are present, the total sum is returned
  *
- * @param h           : the handle to get the offset from
+ * @param h           : the handle to get the data from
  * @param key         : the key to be searched
  * @param size        : the address of a size_t where the size will be set
  * @return            0 if OK, integer value on error
@@ -755,7 +756,7 @@ int grib_get_size(const grib_handle* h, const char* key, size_t* size);
 /**
  *  Get the length of the string representation of the key, if several keys of the same name are present, the maximum length is returned
  *
- * @param h           : the handle to get the offset from
+ * @param h           : the handle to get the data from
  * @param key         : the key to be searched
  * @param length      : the address of a size_t where the length will be set
  * @return            0 if OK, integer value on error
@@ -814,13 +815,14 @@ int grib_get_float_elements(const grib_handle* h, const char* key, const int* in
  *  Get a string value from a key, if several keys of the same name are present, the last one is returned
  * @see  grib_set_string
  *
- * @param h           : the handle to get the data from
- * @param key         : the key to be searched
- * @param mesg       : the address of a string where the data will be retrieved
- * @param length      : the address of a size_t that contains allocated length of the string on input, and that contains the actual length of the string on output
- * @return            0 if OK, integer value on error
+ * @param h        : the handle to get the data from
+ * @param key      : the key to be searched
+ * @param value    : the address of a string where the data will be retrieved
+ * @param length   : the address of a size_t that contains allocated length of the string on input,
+ *                   and that contains the actual length of the string on output
+ * @return         0 if OK, integer value on error
  */
-int grib_get_string(const grib_handle* h, const char* key, char* mesg, size_t* length);
+int grib_get_string(const grib_handle* h, const char* key, char* value, size_t* length);
 
 /**
  *  Get string array values from a key. If several keys of the same name are present, the last one is returned
@@ -911,11 +913,12 @@ int grib_set_double(grib_handle* h, const char* key, double val);
  *
  * @param h           : the handle to set the data to
  * @param key         : the key to be searched
- * @param mesg       : the address of a string where the data will be read
- * @param length      : the address of a size_t that contains the length of the string on input, and that contains the actual packed length of the string on output
+ * @param value       : the address of a string where the data will be read
+ * @param length      : the address of a size_t that contains the length of the string on input,
+ *                      and that contains the actual packed length of the string on output
  * @return            0 if OK, integer value on error
  */
-int grib_set_string(grib_handle* h, const char* key, const char* mesg, size_t* length);
+int grib_set_string(grib_handle* h, const char* key, const char* value, size_t* length);
 
 /**
  *  Set a bytes array from a key. If several keys of the same name are present, the last one is set
@@ -1142,7 +1145,7 @@ void grib_gribex_mode_on(grib_context* c);
  *
  * @param c           : the context
  */
-int grib_get_gribex_mode(grib_context* c);
+int grib_get_gribex_mode(const grib_context* c);
 
 /**
  *  Set the GRIBEX mode off.
@@ -1168,6 +1171,9 @@ void grib_context_set_definitions_path(grib_context* c, const char* path);
  */
 void grib_context_set_samples_path(grib_context* c, const char* path);
 
+void grib_context_set_debug(grib_context* c, int mode);
+void grib_context_set_data_quality_checks(grib_context* c, int val);
+
 /**
  *  Sets memory procedures of the context
  *
@@ -1256,6 +1262,8 @@ long grib_get_api_version(void);
  */
 const char* grib_get_git_sha1(void);
 
+const char* grib_get_git_branch(void);
+
 /**
  *  Get the package name
  *
@@ -1658,7 +1666,7 @@ Error codes returned by the grib_api functions.
 #define GRIB_WRONG_STEP_UNIT		-26
 /** Invalid file id */
 #define GRIB_INVALID_FILE		-27
-/** Invalid grib id */
+/** Invalid GRIB id */
 #define GRIB_INVALID_GRIB		-28
 /** Invalid index id */
 #define GRIB_INVALID_INDEX		-29
