@@ -171,23 +171,9 @@ extern int pthread_mutexattr_settype(pthread_mutexattr_t* attr, int type);
         if (!(a)) codes_assertion_failed(#a, __FILE__, __LINE__); \
     } while (0)
 
-#ifdef __gnu_hurd__
-#define COMPILE_TIME_ASSERT(condition) \
-    extern int compile_time_assert[!!(condition)-1]
-#else
-/* Compile time assertion - Thanks to Ralf Holly */
-#define COMPILE_TIME_ASSERT(condition)        \
-    do {                                      \
-        enum                                  \
-        {                                     \
-            assert_static__ = 1 / (condition) \
-        };                                    \
-    } while (0)
-#endif
-
 #ifdef DEBUG
-#define DebugAssert(a) Assert(a)
-#define DebugAssertAccess(array, index, size)                                                                             \
+#define DEBUG_ASSERT(a) Assert(a)
+#define DEBUG_ASSERT_ACCESS(array, index, size)                                                                             \
     do {                                                                                                                  \
         if (!((index) >= 0 && (index) < (size))) {                                                                        \
             printf("ARRAY ACCESS ERROR: array=%s idx=%ld size=%ld @ %s +%d \n", #array, index, size, __FILE__, __LINE__); \
@@ -195,40 +181,33 @@ extern int pthread_mutexattr_settype(pthread_mutexattr_t* attr, int type);
         }                                                                                                                 \
     } while (0)
 #else
-#define DebugAssert(a)
-#define DebugAssertAccess(array, index, size)
+#define DEBUG_ASSERT(a)
+#define DEBUG_ASSERT_ACCESS(array, index, size)
 #endif
 
 /* Return true if two strings are equal */
 #define STR_EQUAL(a, b) (strcmp((a), (b)) == 0)
+/* Return true if two strings are equal, ignoring case */
+#define STR_EQUAL_NOCASE(a, b) (strcmp_nocase((a), (b)) == 0)
 
 #include "grib_api.h"
 
-#define GRIB_UNKNOWN_VALUE -9999.999
-#define GRIB_KEY_UNDEF "undef"
-
-#define GRIB_HANDLE_BIG_ECMWF_GRIB1 1
-
 #define MAX_ACCESSOR_ATTRIBUTES 20
 #define MAX_FILE_HANDLES_WITH_MULTI 10
 #define ACCESSORS_ARRAY_SIZE 5000
 #define MAX_NUM_CONCEPTS 2000
 #define MAX_NUM_HASH_ARRAY 2000
 
-#define GRIB_NAMESPACE 10
+#define CODES_NAMESPACE   10
 #define MAX_NAMESPACE_LEN 64
 
-#define GRIB_MY_BUFFER 0
-#define GRIB_USER_BUFFER 1
+#define CODES_MY_BUFFER   0
+#define CODES_USER_BUFFER 1
 
-#define GRIB_REAL_MODE4 4
-#define GRIB_REAL_MODE8 8
+#define CODES_REAL_MODE8 8
 
 #define MAX_NUM_SECTIONS 12
 
-#define GRIB_DISPOSABLE_MEMORY 0
-#define GRIB_LONG_LASTING_MEMORY 1
-
 #define GRIB_LOG_PERROR (1 << 10)
 
 #define GRIB_HASH_ARRAY_TYPE_UNKNOWN 0
@@ -236,17 +215,17 @@ extern int pthread_mutexattr_settype(pthread_mutexattr_t* attr, int type);
 #define GRIB_HASH_ARRAY_TYPE_DOUBLE 2
 #define GRIB_HASH_ARRAY_TYPE_STRING 3
 
-#define CODES_GRIB 1
-#define CODES_BUFR 2
+#define CODES_GRIB  1
+#define CODES_BUFR  2
 #define CODES_METAR 3
-#define CODES_GTS 4
+#define CODES_GTS   4
 
 #define CODES_BUFR_UNPACK_STRUCTURE 0
-#define CODES_BUFR_UNPACK_FLAT 1
-#define CODES_BUFR_NEW_DATA 2
+#define CODES_BUFR_UNPACK_FLAT      1
+#define CODES_BUFR_NEW_DATA         2
 
 #define MAX_SMART_TABLE_COLUMNS 20
-#define MAX_CODETABLE_ENTRIES 65536
+#define MAX_CODETABLE_ENTRIES   65536
 
 /* ACCESSOR COMPARE FLAGS */
 #define GRIB_COMPARE_NAMES (1 << 0)
@@ -261,27 +240,23 @@ typedef struct grib_action_file grib_action_file;
 typedef struct grib_action_file_list grib_action_file_list;
 typedef struct grib_block_of_accessors grib_block_of_accessors;
 typedef struct grib_buffer grib_buffer;
-typedef struct grib_accessor_class grib_accessor_class;
+class grib_accessor_class;
 typedef struct grib_action grib_action;
 typedef struct grib_action_class grib_action_class;
 typedef struct grib_section grib_section;
-typedef struct grib_packer grib_packer;
 typedef struct grib_codetable grib_codetable;
 typedef struct grib_smart_table grib_smart_table;
 
-typedef struct grib_accessor grib_accessor;
+class grib_accessor;
 typedef struct grib_iterator_class grib_iterator_class;
 typedef struct grib_nearest_class grib_nearest_class;
-typedef struct grib_box_class grib_box_class;
 typedef struct grib_dumper grib_dumper;
 typedef struct grib_dumper_class grib_dumper_class;
 typedef struct grib_dependency grib_dependency;
-typedef struct string_feed string_feed;
 
 typedef struct codes_condition codes_condition;
 
-/* typedef void           (*dynamic_key_proc)              (const char*, void*)
-*/
+/* typedef void (*dynamic_key_proc) (const char*, void*) */
 typedef void (*nearest_init_class_proc)(grib_nearest_class*);
 typedef int (*nearest_init_proc)(grib_nearest* i, grib_handle*, grib_arguments*);
 
@@ -292,11 +267,6 @@ typedef int (*nearest_find_proc)(grib_nearest* nearest, grib_handle* h,
                                  double* distances, int* indexes, size_t* len);
 typedef int (*nearest_destroy_proc)(grib_nearest* nearest);
 
-typedef void (*box_init_class_proc)(grib_box_class*);
-typedef int (*box_destroy_proc)(grib_box*);
-typedef int (*box_init_proc)(grib_box*, grib_handle*, grib_arguments*);
-typedef grib_points* (*box_get_points_proc)(grib_box*, double, double, double, double, int*);
-
 typedef void (*iterator_init_class_proc)(grib_iterator_class*);
 typedef int (*iterator_init_proc)(grib_iterator* i, grib_handle*, grib_arguments*);
 
@@ -309,54 +279,6 @@ typedef long (*iterator_has_next_proc)(grib_iterator* i);
 typedef int (*grib_pack_proc)(grib_handle* h, const double* in, size_t inlen, void* out, size_t* outlen);
 typedef int (*grib_unpack_proc)(grib_handle* h, const void* in, size_t inlen, double* out, size_t* outlen);
 
-typedef void (*accessor_destroy_proc)(grib_context*, grib_accessor*);
-
-typedef int (*accessor_unpack_long_proc)(grib_accessor*, long*, size_t* len);
-typedef int (*accessor_unpack_double_proc)(grib_accessor*, double*, size_t* len);
-typedef int (*accessor_unpack_float_proc)(grib_accessor*, float*, size_t* len);
-
-typedef int (*accessor_unpack_double_element_proc)(grib_accessor*, size_t, double*);
-typedef int (*accessor_unpack_float_element_proc)(grib_accessor*, size_t, float*);
-typedef int (*accessor_unpack_double_element_set_proc)(grib_accessor*, const size_t*, size_t, double*);
-typedef int (*accessor_unpack_float_element_set_proc)(grib_accessor*, const size_t*, size_t, float*);
-
-typedef int (*accessor_unpack_double_subarray_proc)(grib_accessor*, double*, size_t, size_t);
-typedef int (*accessor_unpack_string_proc)(grib_accessor*, char*, size_t* len);
-typedef int (*accessor_unpack_string_array_proc)(grib_accessor*, char**, size_t* len);
-typedef int (*accessor_unpack_bytes_proc)(grib_accessor*, unsigned char*, size_t* len);
-typedef int (*accessor_get_native_type_proc)(grib_accessor*);
-typedef int (*accessor_notify_change_proc)(grib_accessor*, grib_accessor*);
-typedef void (*accessor_update_size_proc)(grib_accessor*, size_t);
-typedef size_t (*accessor_preferred_size_proc)(grib_accessor*, int);
-typedef void (*accessor_resize_proc)(grib_accessor*, size_t);
-
-typedef grib_accessor* (*accessor_next_proc)(grib_accessor*, int);
-typedef grib_section* (*accessor_sub_section_proc)(grib_accessor*);
-
-
-typedef int (*accessor_pack_missing_proc)(grib_accessor*);
-typedef int (*accessor_pack_is_missing_proc)(grib_accessor*);
-typedef int (*accessor_pack_long_proc)(grib_accessor*, const long*, size_t* len);
-typedef int (*accessor_pack_double_proc)(grib_accessor*, const double*, size_t* len);
-typedef int (*accessor_pack_float_proc)(grib_accessor*, const float*, size_t* len);
-typedef int (*accessor_pack_string_proc)(grib_accessor*, const char*, size_t* len);
-typedef int (*accessor_pack_string_array_proc)(grib_accessor*, const char**, size_t* len);
-typedef int (*accessor_pack_bytes_proc)(grib_accessor*, const unsigned char*, size_t* len);
-typedef int (*accessor_pack_expression_proc)(grib_accessor*, grib_expression*);
-typedef int (*accessor_clear_proc)(grib_accessor*);
-typedef grib_accessor* (*accessor_clone_proc)(grib_accessor*, grib_section*, int*);
-
-typedef void (*accessor_init_class_proc)(grib_accessor_class*);
-
-typedef int (*accessor_compare_proc)(grib_accessor*, grib_accessor*);
-typedef size_t (*accessor_string_proc)(grib_accessor*);
-typedef int (*accessor_value_with_ret_proc)(grib_accessor*, long*);
-typedef long (*accessor_value_proc)(grib_accessor*);
-typedef void (*accessor_dump_proc)(grib_accessor*, grib_dumper*);
-typedef void (*accessor_init_proc)(grib_accessor*, const long len, grib_arguments*);
-typedef void (*accessor_post_init_proc)(grib_accessor*);
-
-typedef int (*accessor_nearest_proc)(grib_accessor*, double, double*);
 
 typedef long (*grib_binop_long_proc)(long, long);
 typedef long (*grib_unop_long_proc)(long);
@@ -366,10 +288,6 @@ typedef double (*grib_unop_double_proc)(double);
 
 typedef int (*grib_binop_string_proc)(char*, char*);
 
-typedef struct second_order_packed second_order_packed;
-typedef void grib_expression_visit_proc(void* udata, grib_expression* e);
-
-
 struct grib_key_value_list
 {
     const char* name;
@@ -384,8 +302,7 @@ struct grib_key_value_list
     grib_key_value_list* next;
 };
 
-
-struct second_order_packed
+/* struct second_order_packed
 {
     unsigned long nbits_per_widths;
     unsigned long nbits_per_group_size;
@@ -394,24 +311,9 @@ struct second_order_packed
     unsigned long* array_of_group_size;
     unsigned long* array_of_group_width;
     long* array_of_group_refs;
-};
-
-/**
-*  an grib_compression
-*  Structure supporting the packing and unpacking procedures
-*
-*  @see  grib_action_create_data
-*/
-struct grib_packer
-{
-    const char* name;
-    grib_pack_proc pack;     /** <  packing procedure                    */
-    grib_unpack_proc unpack; /** < unpacking procedure                    */
-};
-
+}; */
 
 /* --------------- */
-
 typedef struct grib_loader grib_loader;
 typedef int (*grib_loader_init_accessor_proc)(grib_loader*, grib_accessor*, grib_arguments*);
 typedef int (*grib_loader_lookup_long_proc)(grib_context*, grib_loader*, const char* name, long* value);
@@ -426,7 +328,7 @@ struct grib_loader
 };
 
 /**
-*  an action
+*  An action
 *  Structure supporting the creation of accessor, resulting of a statement during a definition file parsing
 *
 *  @see  grib_action_class
@@ -446,17 +348,7 @@ struct grib_action
     char* debug_info; /** purely for debugging and tracing */
 };
 
-typedef struct grib_accessors_list grib_accessors_list;
-
-struct grib_accessors_list
-{
-    grib_accessor* accessor;
-    int rank;
-    grib_accessors_list* next;
-    grib_accessors_list* prev;
-    grib_accessors_list* last;
-};
-
+class grib_accessors_list;
 
 typedef int (*action_create_accessors_handle_proc)(grib_section* p, grib_action* a, grib_loader* h);
 typedef int (*action_notify_change_proc)(grib_action* a, grib_accessor* observer, grib_accessor* observed);
@@ -470,7 +362,7 @@ typedef grib_action* (*action_reparse_proc)(grib_action* a, grib_accessor*, int*
 typedef int (*action_execute_proc)(grib_action* a, grib_handle*);
 
 /**
-*  an action_class
+*  An action_class
 *  Structure supporting the specific behaviour of an action
 *
 *  @see  grib_action
@@ -496,24 +388,23 @@ struct grib_action_class
     action_execute_proc execute;
 };
 
-
 /**
-*  a buffer
+*  A buffer
 *  Structure containing the data of a message
 */
 struct grib_buffer
 {
-    int property;        /** < property parameter of buffer         */
-    int validity;        /** < validity parameter of buffer         */
-    int growable;        /** < buffer can be grown                  */
-    size_t length;       /** < Buffer length                        */
-    size_t ulength;      /** < length used of the buffer            */
-    size_t ulength_bits; /** < length used of the buffer in bits  */
-    unsigned char* data; /** < the data byte array                  */
+    int property;        /** < property parameter of buffer */
+    int validity;        /** < validity parameter of buffer */
+    int growable;        /** < buffer can be grown */
+    size_t length;       /** < Buffer length */
+    size_t ulength;      /** < length used of the buffer */
+    size_t ulength_bits; /** < length used of the buffer in bits */
+    unsigned char* data; /** < the data byte array */
 };
 
 /**
-*  an Accessor
+*  An accessor
 *  Structure supporting each single data unit and allowing its access
 *  @see  grib_accessor_class
 */
@@ -532,56 +423,28 @@ struct grib_virtual_value
     int type;
 };
 
-struct grib_accessor
-{
-    const char* name;       /** < name of the accessor                       */
-    const char* name_space; /** < namespace to which the accessor belongs    */
-    grib_context* context;
-    grib_handle* h;
-    grib_action* creator;        /** < action that created the accessor           */
-    long length;                 /** < byte length of the accessor                */
-    long offset;                 /** < offset of the data in the buffer           */
-    grib_section* parent;        /** < section to which the accessor is attached  */
-    grib_accessor* next;         /** < next accessor in list                      */
-    grib_accessor* previous;     /** < next accessor in list                      */
-    grib_accessor_class* cclass; /** < behaviour of the accessor                  */
-    unsigned long flags;         /** < Various flags                              */
-    grib_section* sub_section;
-
-    const char* all_names[MAX_ACCESSOR_NAMES];       /** < name of the accessor */
-    const char* all_name_spaces[MAX_ACCESSOR_NAMES]; /** < namespace to which the accessor belongs */
-    int dirty;
-
-    grib_accessor* same;        /** < accessors with the same name */
-    long loop;                  /** < used in lists */
-    long bufr_subset_number;    /** < bufr subset (bufr data accessors belong to different subsets)*/
-    long bufr_group_number;     /** < used in bufr */
-    grib_virtual_value* vvalue; /** < virtual value used when transient flag on **/
-    const char* set;
-    grib_accessor* attributes[MAX_ACCESSOR_ATTRIBUTES]; /** < attributes are accessors */
-    grib_accessor* parent_as_attribute;
-};
-
-#define GRIB_ACCESSOR_FLAG_READ_ONLY (1 << 1)
-#define GRIB_ACCESSOR_FLAG_DUMP (1 << 2)
+#define GRIB_ACCESSOR_FLAG_READ_ONLY        (1 << 1)
+#define GRIB_ACCESSOR_FLAG_DUMP             (1 << 2)
 #define GRIB_ACCESSOR_FLAG_EDITION_SPECIFIC (1 << 3)
-#define GRIB_ACCESSOR_FLAG_CAN_BE_MISSING (1 << 4)
-#define GRIB_ACCESSOR_FLAG_HIDDEN (1 << 5)
-#define GRIB_ACCESSOR_FLAG_CONSTRAINT (1 << 6)
-#define GRIB_ACCESSOR_FLAG_BUFR_DATA (1 << 7)
-#define GRIB_ACCESSOR_FLAG_NO_COPY (1 << 8)
-#define GRIB_ACCESSOR_FLAG_COPY_OK (1 << 9)
-#define GRIB_ACCESSOR_FLAG_FUNCTION (1 << 10)
-#define GRIB_ACCESSOR_FLAG_DATA (1 << 11)
-#define GRIB_ACCESSOR_FLAG_NO_FAIL (1 << 12)
-#define GRIB_ACCESSOR_FLAG_TRANSIENT (1 << 13)
-#define GRIB_ACCESSOR_FLAG_STRING_TYPE (1 << 14)
-#define GRIB_ACCESSOR_FLAG_LONG_TYPE (1 << 15)
-#define GRIB_ACCESSOR_FLAG_DOUBLE_TYPE (1 << 16)
-#define GRIB_ACCESSOR_FLAG_LOWERCASE (1 << 17)
+#define GRIB_ACCESSOR_FLAG_CAN_BE_MISSING   (1 << 4)
+#define GRIB_ACCESSOR_FLAG_HIDDEN           (1 << 5)
+#define GRIB_ACCESSOR_FLAG_CONSTRAINT       (1 << 6)
+#define GRIB_ACCESSOR_FLAG_BUFR_DATA        (1 << 7)
+#define GRIB_ACCESSOR_FLAG_NO_COPY          (1 << 8)
+#define GRIB_ACCESSOR_FLAG_COPY_OK          (1 << 9)
+#define GRIB_ACCESSOR_FLAG_FUNCTION         (1 << 10)
+#define GRIB_ACCESSOR_FLAG_DATA             (1 << 11)
+#define GRIB_ACCESSOR_FLAG_NO_FAIL          (1 << 12)
+#define GRIB_ACCESSOR_FLAG_TRANSIENT        (1 << 13)
+#define GRIB_ACCESSOR_FLAG_STRING_TYPE      (1 << 14)
+#define GRIB_ACCESSOR_FLAG_LONG_TYPE        (1 << 15)
+#define GRIB_ACCESSOR_FLAG_DOUBLE_TYPE      (1 << 16)
+#define GRIB_ACCESSOR_FLAG_LOWERCASE        (1 << 17)
+#define GRIB_ACCESSOR_FLAG_BUFR_COORD       (1 << 18)
+#define GRIB_ACCESSOR_FLAG_COPY_IF_CHANGING_EDITION (1 << 19)
 
 /**
-*  a section accessor
+*  A section accessor
 *  Structure supporting hierarchical naming of the accessors
 *  @see  grib_accessor
 */
@@ -589,30 +452,26 @@ struct grib_section
 {
     grib_accessor* owner;
     grib_handle* h;                 /** < Handles of all accessors and buffer  */
-    grib_accessor* aclength;        /** < block of the length of the block     */
-    grib_block_of_accessors* block; /** < block                                */
-    grib_action* branch;            /** < branch that created the block        */
+    grib_accessor* aclength;        /** < block of the length of the block */
+    grib_block_of_accessors* block; /** < block */
+    grib_action* branch;            /** < branch that created the block */
     size_t length;
     size_t padding;
 };
 
-
 struct grib_iterator_class
 {
     grib_iterator_class** super;
     const char* name;
     size_t size;
-
     int inited;
     iterator_init_class_proc init_class;
-
-    iterator_init_proc init;
-    iterator_destroy_proc destroy;
-
-    iterator_next_proc next;
-    iterator_previous_proc previous;
-    iterator_reset_proc reset;
-    iterator_has_next_proc has_next;
+    iterator_init_proc       init;
+    iterator_destroy_proc    destroy;
+    iterator_next_proc       next;
+    iterator_previous_proc   previous;
+    iterator_reset_proc      reset;
+    iterator_has_next_proc   has_next;
 };
 
 struct grib_nearest_class
@@ -620,33 +479,14 @@ struct grib_nearest_class
     grib_nearest_class** super;
     const char* name;
     size_t size;
-
     int inited;
     nearest_init_class_proc init_class;
-
-    nearest_init_proc init;
-    nearest_destroy_proc destroy;
-
-    nearest_find_proc find;
-};
-
-struct grib_box_class
-{
-    grib_box_class** super;
-    const char* name;
-    size_t size;
-    int inited;
-    box_init_class_proc init_class;
-    box_init_proc init;
-    box_destroy_proc destroy;
-    box_get_points_proc get_points;
+    nearest_init_proc       init;
+    nearest_destroy_proc    destroy;
+    nearest_find_proc       find;
 };
 
 /* --------------- */
-/* --------------- */
-typedef void (*search_all_callback_proc)(grib_accessor*, void* data);
-/* --------------- */
-
 typedef int (*dumper_init_proc)(grib_dumper*);
 typedef void (*dumper_dump_proc)(grib_dumper*, grib_accessor*, const char* comment);
 typedef void (*dumper_dump_section_proc)(grib_dumper*, grib_accessor*, grib_block_of_accessors* block);
@@ -691,18 +531,18 @@ struct grib_dumper_class
 
 struct grib_iterator
 {
-    grib_arguments* args; /**  args of iterator   */
+    grib_arguments* args; /**  args of iterator */
     grib_handle* h;
-    long e;       /**  current element    */
-    size_t nv;    /**  number of values   */
-    double* data; /**  data values        */
+    long e;       /**  current element */
+    size_t nv;    /**  number of values */
+    double* data; /**  data values */
     grib_iterator_class* cclass;
     unsigned long flags;
 };
 
 struct grib_nearest
 {
-    grib_arguments* args; /**  args of iterator   */
+    grib_arguments* args; /**  args of iterator */
     grib_handle* h;
     grib_context* context;
     double* values;
@@ -711,16 +551,6 @@ struct grib_nearest
     unsigned long flags;
 };
 
-struct grib_box
-{
-    grib_box_class* cclass;
-    grib_context* context;
-    grib_arguments* args;
-    grib_handle* h;
-    unsigned long flags;
-    grib_points* points;
-};
-
 struct grib_dependency
 {
     grib_dependency* next;
@@ -741,7 +571,6 @@ typedef struct grib_trie_with_rank_list grib_trie_with_rank_list;
 typedef struct grib_trie_with_rank grib_trie_with_rank;
 typedef struct grib_itrie grib_itrie;
 
-
 /* Dynamic array of strings */
 struct grib_sarray
 {
@@ -878,7 +707,6 @@ struct codes_condition
     double rightDouble;
 };
 
-
 void codes_assertion_failed(const char* message, const char* file, int line);
 
 #define MAX_SET_VALUES 10
@@ -886,11 +714,9 @@ void codes_assertion_failed(const char* message, const char* file, int line);
 
 struct grib_handle
 {
-    grib_context* context;         /** < context attached to this handle    */
-    grib_buffer* buffer;           /** < buffer attached to the handle      */
-    grib_section* root;            /**  the root      section*/
-    grib_section* asserts;         /** the assertion section*/
-    grib_section* rules;           /** the rules     section*/
+    grib_context* context;         /** < context attached to this handle */
+    grib_buffer* buffer;           /** < buffer attached to the handle */
+    grib_section* root;            /**  the root section*/
     grib_dependency* dependencies; /** List of dependencies */
     grib_handle* main;             /** Used during reparsing */
     grib_handle* kid;              /** Used during reparsing */
@@ -898,7 +724,6 @@ struct grib_handle
     int values_stack;
     const grib_values* values[MAX_SET_VALUES]; /** Used when setting multiple values at once */
     size_t values_count[MAX_SET_VALUES];       /** Used when setting multiple values at once */
-    int dont_trigger;                          /** Don't notify triggers */
     int partial;                               /** Not a complete message (just headers) */
     int header_mode;                           /** Header not jet complete */
     char* gts_header;
@@ -910,92 +735,22 @@ struct grib_handle
     char* section_length[MAX_NUM_SECTIONS];
     int sections_count;
     off_t offset;
-    long bufr_subset_number; /* bufr subset number */
-    long bufr_group_number;  /* used in bufr */
     /* grib_accessor* groups[MAX_NUM_GROUPS]; */
-    long missingValueLong;
-    double missingValueDouble;
     ProductKind product_kind;
     /* grib_trie* bufr_elements_table; */
 };
 
+/* For GRIB2 multi-field messages */
 struct grib_multi_handle
 {
-    grib_context* context; /** < context attached to this handle  */
-    grib_buffer* buffer;   /** < buffer attached to the handle    */
+    grib_context* context; /** < context attached to this handle */
+    grib_buffer* buffer;   /** < buffer attached to the handle */
     size_t offset;
     size_t length;
 };
 
 
-struct grib_accessor_class
-{
-    grib_accessor_class** super;
-    const char* name;
-    size_t size;
-
-    int inited;
-    accessor_init_class_proc init_class;
-
-    accessor_init_proc init;
-    accessor_post_init_proc post_init;
-    accessor_destroy_proc destroy;
-
-    accessor_dump_proc dump;
-    accessor_value_proc next_offset;
-
-    accessor_string_proc string_length;
-    accessor_value_with_ret_proc value_count;
-
-    accessor_value_proc byte_count;
-    accessor_value_proc byte_offset;
-
-    accessor_get_native_type_proc get_native_type;
-
-    accessor_sub_section_proc sub_section;
-
-    accessor_pack_missing_proc pack_missing;
-    accessor_pack_is_missing_proc is_missing;
-
-    accessor_pack_long_proc pack_long;
-    accessor_unpack_long_proc unpack_long;
-
-    accessor_pack_double_proc pack_double;
-    accessor_pack_float_proc pack_float;
-    accessor_unpack_double_proc unpack_double;
-    accessor_unpack_float_proc unpack_float;
-
-    accessor_pack_string_proc pack_string;
-    accessor_unpack_string_proc unpack_string;
-
-    accessor_pack_string_array_proc pack_string_array;
-    accessor_unpack_string_array_proc unpack_string_array;
-
-    accessor_pack_bytes_proc pack_bytes;
-    accessor_unpack_bytes_proc unpack_bytes;
-
-    accessor_pack_expression_proc pack_expression;
-
-    accessor_notify_change_proc notify_change;
-    accessor_update_size_proc update_size;
-
-    accessor_preferred_size_proc preferred_size;
-    accessor_resize_proc resize;
-
-    accessor_nearest_proc nearest_smaller_value;
-    accessor_next_proc next;
-    accessor_compare_proc compare;
-    accessor_unpack_double_element_proc unpack_double_element;
-    accessor_unpack_float_element_proc unpack_float_element;
-    accessor_unpack_double_element_set_proc unpack_double_element_set;
-    accessor_unpack_float_element_set_proc unpack_float_element_set;
-    accessor_unpack_double_subarray_proc unpack_double_subarray;
-    accessor_clear_proc clear;
-    accessor_clone_proc make_clone;
-};
-
 typedef struct grib_multi_support grib_multi_support;
-
 struct grib_multi_support
 {
     FILE* file;
@@ -1005,14 +760,13 @@ struct grib_multi_support
     unsigned char* sections[8];
     unsigned char* bitmap_section;
     size_t bitmap_section_length;
-    size_t sections_length[9];
+    size_t sections_length[9]; /* GRIB2 has 9 sections */
     int section_number;
     grib_multi_support* next;
 };
 
 /* Hash_array */
 typedef struct grib_hash_array_value grib_hash_array_value;
-
 struct grib_hash_array_value
 {
     grib_hash_array_value* next;
@@ -1025,7 +779,6 @@ struct grib_hash_array_value
 
 /* Concepts */
 typedef struct grib_concept_condition grib_concept_condition;
-
 struct grib_concept_condition
 {
     grib_concept_condition* next;
@@ -1042,7 +795,6 @@ struct grib_concept_value_name
 };
 
 typedef struct grib_concept_value grib_concept_value;
-
 struct grib_concept_value
 {
     grib_concept_value* next;
@@ -1062,6 +814,7 @@ struct grib_context
     int no_big_group_split;
     int no_spd;
     int keep_matrix;
+    int grib_hourly_steps_with_units;
     char* grib_definition_files_path;
     char* grib_samples_path;
     char* grib_concept_path;
@@ -1118,6 +871,7 @@ struct grib_context
     int bufr_set_to_missing_if_out_of_range;
     int bufr_multi_element_constant_arrays;
     int grib_data_quality_checks;
+    int single_precision;
     FILE* log_stream;
     grib_trie* classes;
     grib_trie* lists;
@@ -1137,7 +891,7 @@ typedef int (*expression_evaluate_double_proc)(grib_expression*, grib_handle*, d
 typedef const char* (*expression_evaluate_string_proc)(grib_expression*, grib_handle*, char*, size_t*, int*);
 typedef const char* (*expression_get_name_proc)(grib_expression*);
 
-typedef void (*expression_print_proc)(grib_context*, grib_expression*, grib_handle*);
+typedef void (*expression_print_proc)(grib_context*, grib_expression*, grib_handle*, FILE*);
 typedef void (*expression_add_dependency_proc)(grib_expression* e, grib_accessor* observer);
 
 typedef struct grib_expression_class grib_expression_class;
@@ -1159,24 +913,17 @@ struct grib_expression_class
     const char* name;
     size_t size;
     int inited;
-
-    expression_class_init_proc init_class;
-    expression_init_proc init;
-    expression_destroy_proc destroy;
-
-
-    expression_print_proc print;
-    expression_add_dependency_proc add_dependency;
-
-    expression_native_type_proc native_type;
-    expression_get_name_proc get_name;
-
-    expression_evaluate_long_proc evaluate_long;
+    expression_init_proc            init;
+    expression_destroy_proc         destroy;
+    expression_print_proc           print;
+    expression_add_dependency_proc  add_dependency;
+    expression_native_type_proc     native_type;
+    expression_get_name_proc        get_name;
+    expression_evaluate_long_proc   evaluate_long;
     expression_evaluate_double_proc evaluate_double;
     expression_evaluate_string_proc evaluate_string;
 };
 
-
 struct grib_arguments
 {
     struct grib_arguments* next;
@@ -1278,7 +1025,6 @@ struct grib_int_array
     int* el;
 };
 
-#if 1
 struct grib_fieldset
 {
     grib_context* context;
@@ -1293,69 +1039,6 @@ struct grib_fieldset
     long current;
     grib_field** fields;
 };
-#endif
-
-#if 0
-/* grib db */
-struct grib_db
-{
-    grib_context* context;
-    size_t size;
-    size_t fields_array_size;
-    grib_column* columns;
-    size_t columns_size;
-    grib_field** fields;
-};
-
-struct grib_fieldset
-{
-    grib_context* context;
-    grib_db* db;
-    grib_int_array* filter;
-    grib_int_array* order;
-    size_t size;
-    grib_query* query;
-    long current;
-};
-#endif
-
-/* concept index structures */
-
-typedef struct grib_concept_index_key grib_concept_index_key;
-typedef struct grib_concept_index grib_concept_index;
-typedef struct grib_conditions_tree grib_conditions_tree;
-typedef struct grib_concept_entry grib_concept_entry;
-typedef struct grib_concept_key grib_concept_key;
-
-struct grib_concept_index_entry
-{
-    char* name;
-    char* value;
-    int type;
-    grib_concept_entry* next;
-};
-
-struct grib_concept_index_key
-{
-    char* name;
-    int type;
-    grib_concept_key* next;
-};
-
-struct grib_concept_index
-{
-    grib_context* context;
-    grib_concept_key* keys;
-    grib_conditions_tree* conditions;
-};
-
-struct grib_conditions_tree
-{
-    char* value;
-    void* object;
-    grib_conditions_tree* next;
-    grib_conditions_tree* next_key;
-};
 
 /* support for in-memory definition and tables */
 
@@ -1397,7 +1080,6 @@ struct grib_field_list
     grib_field_list* next;
 };
 
-
 struct grib_index
 {
     grib_context* context;
@@ -1484,10 +1166,8 @@ struct bufr_keys_iterator
     grib_trie* seen;
 };
 
-/* ----------*/
-/* md5 */
+/* --------- md5 --------*/
 typedef unsigned long cvs_uint32;
-
 struct cvs_MD5Context
 {
     cvs_uint32 buf[4];
@@ -1497,7 +1177,6 @@ struct cvs_MD5Context
 /* --- */
 
 typedef struct grib_rule_entry grib_rule_entry;
-
 struct grib_rule_entry
 {
     grib_rule_entry* next;
@@ -1506,7 +1185,6 @@ struct grib_rule_entry
 };
 
 typedef struct grib_rule grib_rule;
-
 struct grib_rule
 {
     grib_rule* next;
@@ -1515,7 +1193,6 @@ struct grib_rule
 };
 
 typedef struct grib_case grib_case;
-
 struct grib_case
 {
     grib_arguments* values;
@@ -1524,7 +1201,6 @@ struct grib_case
 };
 
 /* ----------*/
-
 typedef struct code_table_entry
 {
     char* abbreviation;
@@ -1543,7 +1219,7 @@ struct grib_codetable
 
 typedef struct grib_smart_table_entry
 {
-    /*int   code;*/
+    /*int code;*/
     char* abbreviation;
     char* column[MAX_SMART_TABLE_COLUMNS];
 } grib_smart_table_entry;
@@ -1609,39 +1285,8 @@ typedef struct j2k_encode_helper
 
 #ifdef __cplusplus
 }
+#include "accessor/grib_accessor.h"
+#include "accessor/grib_accessors_list.h"
 #endif
 
 #endif
-
-
-
-/* This part is automatically generated by ./errors.pl, do not edit */
-#ifndef grib_errors_internal_H
-#define grib_errors_internal_H
-/** Value mismatch */
-#define GRIB_VALUE_MISMATCH 1
-/** double values are different */
-#define GRIB_DOUBLE_VALUE_MISMATCH 2
-/** long values are different */
-#define GRIB_LONG_VALUE_MISMATCH 3
-/** byte values are different */
-#define GRIB_BYTE_VALUE_MISMATCH 4
-/** string values are different */
-#define GRIB_STRING_VALUE_MISMATCH 5
-/** Offset mismatch */
-#define GRIB_OFFSET_MISMATCH 6
-/** Count mismatch */
-#define GRIB_COUNT_MISMATCH 7
-/** Name mismatch */
-#define GRIB_NAME_MISMATCH 8
-/** Type mismatch */
-#define GRIB_TYPE_MISMATCH 9
-/** Type and value mismatch */
-#define GRIB_TYPE_AND_VALUE_MISMATCH 10
-/** Unable to compare accessors */
-#define GRIB_UNABLE_TO_COMPARE_ACCESSORS 11
-/** Unable to reset iterator */
-#define GRIB_UNABLE_TO_RESET_ITERATOR 12
-/** Assertion failure */
-#define GRIB_ASSERTION_FAILURE 13
-#endif
