@@ -11,9 +11,15 @@
 
 const char* grib_get_git_sha1(void)
 {
-    return "219435789423d7d9618daf6c25adcb9d9da37621";
+    return "9e6bdfc1cbd60ce12e16b070f6b143c8a48119f2";
 }
+
+const char* grib_get_git_branch(void)
+{
+    return "HEAD";
+}
+
 const char* codes_get_build_date(void)
 {
-    return "2024.09.05";
+    return "2024.09.24";
 }
