@@ -55,7 +55,7 @@ static int grib_fieldset_set_order_by(grib_fieldset* set, grib_order_by* ob);
 
 
 /* --------------- grib_column functions ------------------*/
-int grib_fieldset_new_column(grib_fieldset* set, int id, char* key, int type)
+static int grib_fieldset_new_column(grib_fieldset* set, int id, char* key, int type)
 {
     grib_column* column = NULL;
     grib_context* c;
@@ -73,8 +73,7 @@ int grib_fieldset_new_column(grib_fieldset* set, int id, char* key, int type)
             set->columns[id].long_values = (long*)grib_context_malloc_clear(c, sizeof(long) * GRIB_START_ARRAY_SIZE);
             if (!set->columns[id].long_values) {
                 grib_context_log(c, GRIB_LOG_ERROR,
-                                 "grib_fieldset_new_column: Cannot malloc %ld bytes",
-                                 sizeof(long) * GRIB_START_ARRAY_SIZE);
+                                 "%s: Error allocating %zu bytes", __func__, sizeof(long) * GRIB_START_ARRAY_SIZE);
                 err = GRIB_OUT_OF_MEMORY;
                 return err;
             }
@@ -83,8 +82,7 @@ int grib_fieldset_new_column(grib_fieldset* set, int id, char* key, int type)
             set->columns[id].double_values = (double*)grib_context_malloc_clear(c, sizeof(double) * GRIB_START_ARRAY_SIZE);
             if (!set->columns[id].double_values) {
                 grib_context_log(c, GRIB_LOG_ERROR,
-                                 "grib_fieldset_new_column: Cannot malloc %ld bytes",
-                                 sizeof(double) * GRIB_START_ARRAY_SIZE);
+                                 "%s: Error allocating %zu bytes", __func__, sizeof(double) * GRIB_START_ARRAY_SIZE);
                 err = GRIB_OUT_OF_MEMORY;
                 return err;
             }
@@ -93,8 +91,7 @@ int grib_fieldset_new_column(grib_fieldset* set, int id, char* key, int type)
             set->columns[id].string_values = (char**)grib_context_malloc_clear(c, sizeof(char*) * GRIB_START_ARRAY_SIZE);
             if (!set->columns[id].string_values) {
                 grib_context_log(c, GRIB_LOG_ERROR,
-                                 "grib_fieldset_new_column: Cannot malloc %ld bytes",
-                                 sizeof(char*) * GRIB_START_ARRAY_SIZE);
+                                 "%s: Error allocating %zu bytes", __func__, sizeof(char*) * GRIB_START_ARRAY_SIZE);
                 err = GRIB_OUT_OF_MEMORY;
                 return err;
             }
@@ -116,14 +113,12 @@ int grib_fieldset_new_column(grib_fieldset* set, int id, char* key, int type)
 
 static void grib_fieldset_delete_columns(grib_fieldset* set)
 {
-    int i = 0;
-    grib_context* c;
-
     if (!set)
         return;
-    c = set->context;
 
-    for (i = 0; i < set->columns_size; i++) {
+    const grib_context* c = set->context;
+
+    for (size_t i = 0; i < set->columns_size; i++) {
         int j = 0;
         switch (set->columns[i].type) {
             case GRIB_TYPE_LONG:
@@ -149,30 +144,26 @@ static void grib_fieldset_delete_columns(grib_fieldset* set)
 
 static int grib_fieldset_columns_resize(grib_fieldset* set, size_t newsize)
 {
-    double* newdoubles;
-    long* newlongs;
-    char** newstrings;
-    int* newerrors;
-    int i = 0;
-    grib_context* c;
-
     if (!set || !set->columns)
         return GRIB_INVALID_ARGUMENT;
 
-    c = set->context;
+    double* newdoubles = NULL;
+    long* newlongs     = NULL;
+    char** newstrings  = NULL;
+    int* newerrors     = NULL;
+    const grib_context* c = set->context;
 
     if (newsize <= set->columns[0].values_array_size)
         return 0;
 
-    for (i = 0; i < set->columns_size; i++) {
+    for (size_t i = 0; i < set->columns_size; i++) {
         switch (set->columns[i].type) {
             case GRIB_TYPE_LONG:
                 newlongs = (long*)grib_context_realloc(c, set->columns[i].long_values,
                                                        newsize * sizeof(long));
                 if (!newlongs) {
                     grib_context_log(c, GRIB_LOG_ERROR,
-                                     "grib_fieldset_columns_resize: Cannot malloc %ld bytes",
-                                     newsize - set->columns[i].values_array_size);
+                                     "%s: Error allocating %zu bytes", __func__, newsize - set->columns[i].values_array_size);
                     return GRIB_OUT_OF_MEMORY;
                 }
                 else
@@ -183,8 +174,7 @@ static int grib_fieldset_columns_resize(grib_fieldset* set, size_t newsize)
                                                            newsize * sizeof(double));
                 if (!newdoubles) {
                     grib_context_log(c, GRIB_LOG_ERROR,
-                                     "grib_fieldset_columns_resize: Cannot malloc %ld bytes",
-                                     newsize - set->columns[i].values_array_size);
+                                     "%s: Error allocating %zu bytes", __func__, newsize - set->columns[i].values_array_size);
                     return GRIB_OUT_OF_MEMORY;
                 }
                 else
@@ -195,8 +185,7 @@ static int grib_fieldset_columns_resize(grib_fieldset* set, size_t newsize)
                                                           newsize * sizeof(char*));
                 if (!newstrings) {
                     grib_context_log(c, GRIB_LOG_ERROR,
-                                     "grib_fieldset_columns_resize: Cannot malloc %ld bytes",
-                                     newsize - set->columns[i].values_array_size);
+                                     "%s: Error allocating %zu bytes", __func__, newsize - set->columns[i].values_array_size);
                     return GRIB_OUT_OF_MEMORY;
                 }
                 else
@@ -206,8 +195,7 @@ static int grib_fieldset_columns_resize(grib_fieldset* set, size_t newsize)
         newerrors = (int*)grib_context_realloc(c, set->columns[i].errors, newsize * sizeof(int));
         if (!newerrors) {
             grib_context_log(c, GRIB_LOG_ERROR,
-                             "grib_fieldset_columns_resize: Cannot malloc %ld bytes",
-                             newsize * sizeof(int));
+                             "%s: Error allocating %zu bytes", __func__, newsize * sizeof(int));
             return GRIB_OUT_OF_MEMORY;
         }
         else
@@ -332,8 +320,7 @@ static grib_fieldset* grib_fieldset_create_from_keys(grib_context* c, const char
     msize = sizeof(grib_fieldset);
     set   = (grib_fieldset*)grib_context_malloc_clear(c, msize);
     if (!set) {
-        grib_context_log(c, GRIB_LOG_ERROR,
-                         "grib_fieldset_create_from_keys: Cannot malloc %lu bytes", msize);
+        grib_context_log(c, GRIB_LOG_ERROR, "%s: Error allocating %zu bytes", __func__, msize);
         return NULL;
     }
 
@@ -357,7 +344,7 @@ static grib_fieldset* grib_fieldset_create_from_keys(grib_context* c, const char
 
     set->columns = (grib_column*)grib_context_malloc_clear(c, sizeof(grib_column) * nkeys);
     if (!set->columns) {
-        grib_context_log(c, GRIB_LOG_ERROR, "grib_fieldset_create_from_keys: memory allocation error");
+        grib_context_log(c, GRIB_LOG_ERROR, "%s: memory allocation error", __func__);
         *err = GRIB_OUT_OF_MEMORY;
         return NULL;
     }
@@ -411,21 +398,15 @@ static grib_fieldset* grib_fieldset_create_from_order_by(grib_context* c, grib_o
     return set;
 }
 
-/* Experimental: Needs more work */
+// Experimental: Needs more work
 int grib_fieldset_apply_where(grib_fieldset* set, const char* where_string)
 {
-    int err      = GRIB_NOT_IMPLEMENTED;
-    grib_math* m = NULL;
-
-    if (!set)
-        return GRIB_INVALID_ARGUMENT;
-
-    m = grib_math_new(set->context, where_string, &err);
-    if (err || !m) return err;
-
-    print_math(m);
-    printf("\n");
-    grib_math_delete(set->context, m);
+    // grib_math* m = NULL;
+    // m = grib_math_new(set->context, where_string, &err);
+    // if (err || !m) return err;
+    // print_math(m);
+    // printf("\n");
+    // grib_math_delete(set->context, m);
     return GRIB_NOT_IMPLEMENTED;
 }
 
@@ -821,8 +802,7 @@ static int grib_fieldset_resize_int_array(grib_int_array* a, size_t newsize)
     el = (int*)grib_context_realloc(a->context, a->el, newsize);
     if (!el) {
         grib_context_log(a->context, GRIB_LOG_ERROR,
-                         "grib_fieldset_resize_int_array: Cannot malloc %lu bytes",
-                         newsize);
+                         "%s: Error allocating %zu bytes", __func__, newsize);
         return GRIB_OUT_OF_MEMORY;
     }
     else
@@ -833,12 +813,9 @@ static int grib_fieldset_resize_int_array(grib_int_array* a, size_t newsize)
 
 static void grib_fieldset_delete_int_array(grib_int_array* f)
 {
-    grib_context* c = NULL;
-
-    if (!f)
-        return;
-    c = f->context;
+    if (!f) return;
 
+    const grib_context* c = f->context;
     grib_context_free(c, f->el);
     grib_context_free(c, f);
 }
@@ -865,8 +842,7 @@ static int grib_fieldset_resize_fields(grib_fieldset* set, size_t newsize)
     fields = (grib_field**)grib_context_realloc(set->context, set->fields, newsize * sizeof(grib_field*));
     if (!fields) {
         grib_context_log(set->context, GRIB_LOG_ERROR,
-                         "grib_fieldset_resize_fields: Cannot malloc %lu bytes",
-                         newsize * sizeof(grib_field*));
+                         "%s: Error allocating %zu bytes", __func__, newsize * sizeof(grib_field*));
         return GRIB_OUT_OF_MEMORY;
     }
     else
