@@ -14,7 +14,7 @@
 static pthread_once_t once    = PTHREAD_ONCE_INIT;
 static pthread_mutex_t mutex1 = PTHREAD_MUTEX_INITIALIZER;
 static pthread_mutex_t mutex2 = PTHREAD_MUTEX_INITIALIZER;
-static void init()
+static void init_mutex()
 {
     pthread_mutexattr_t attr;
     pthread_mutexattr_init(&attr);
@@ -27,7 +27,7 @@ static void init()
 static int once = 0;
 static omp_nest_lock_t mutex1;
 static omp_nest_lock_t mutex2;
-static void init()
+static void init_mutex()
 {
     GRIB_OMP_CRITICAL(lock_grib_io_c)
     {
@@ -84,46 +84,59 @@ typedef struct reader
 
 } reader;
 
-static int read_the_rest(reader* r, size_t message_length, unsigned char* tmp, int already_read, int check7777)
+// If no_alloc argument is 1, then the content of the message are not stored. We just seek to the final 7777
+static int read_the_rest(reader* r, size_t message_length, unsigned char* tmp, int already_read, int check7777, int no_alloc)
 {
     int err = GRIB_SUCCESS;
     size_t buffer_size;
     size_t rest;
-    unsigned char* buffer;
+    unsigned char* buffer = NULL;
     grib_context* c = grib_context_get_default();
 
     if (message_length == 0)
         return GRIB_BUFFER_TOO_SMALL;
 
-    buffer_size     = message_length;
+    buffer_size     = message_length; // store the whole message in the buffer
+    if (no_alloc)
+        buffer_size = 5; // big enough to store the 7777
     rest            = message_length - already_read;
     r->message_size = message_length;
     buffer          = (unsigned char*)r->alloc(r->alloc_data, &buffer_size, &err);
     if (err)
         return err;
 
-    if (buffer == NULL || (buffer_size < message_length)) {
-        return GRIB_BUFFER_TOO_SMALL;
+    if (no_alloc) {
+        r->seek(r->read_data,  rest - 4); // jump to the end before the 7777
+    } else {
+        if (buffer == NULL || (buffer_size < message_length)) {
+            return GRIB_BUFFER_TOO_SMALL;
+        }
+        memcpy(buffer, tmp, already_read);
     }
 
-    memcpy(buffer, tmp, already_read);
-
-    if ((r->read(r->read_data, buffer + already_read, rest, &err) != rest) || err) {
-        /*fprintf(stderr, "read_the_rest: r->read failed: %s\n", grib_get_error_message(err));*/
+    bool read_failed = false;
+    if (no_alloc) {
+        read_failed = ((r->read(r->read_data, buffer, 4, &err) != 4) || err);
+    }
+    else {
+        read_failed = ((r->read(r->read_data, buffer + already_read, rest, &err) != rest) || err);
+    }
+    if (read_failed) {
         if (c->debug)
-            fprintf(stderr, "ECCODES DEBUG read_the_rest: Read failed (Coded length=%zu, Already read=%d)\n",
-                    message_length, already_read);
+            fprintf(stderr, "ECCODES DEBUG %s: Read failed (Coded length=%zu, Already read=%d)",
+                    __func__, message_length, already_read);
         return err;
     }
 
+    const size_t mlen = no_alloc ? 4 : message_length;
     if (check7777 && !r->headers_only &&
-        (buffer[message_length - 4] != '7' ||
-         buffer[message_length - 3] != '7' ||
-         buffer[message_length - 2] != '7' ||
-         buffer[message_length - 1] != '7'))
+        (buffer[mlen - 4] != '7' ||
+         buffer[mlen - 3] != '7' ||
+         buffer[mlen - 2] != '7' ||
+         buffer[mlen - 1] != '7'))
     {
         if (c->debug)
-            fprintf(stderr, "ECCODES DEBUG read_the_rest: No final 7777 at expected location (Coded length=%zu)\n", message_length);
+            fprintf(stderr, "ECCODES DEBUG %s: No final 7777 at expected location (Coded length=%zu)\n", __func__, message_length);
         return GRIB_WRONG_LENGTH;
     }
 
@@ -144,7 +157,7 @@ static int read_the_rest(reader* r, size_t message_length, unsigned char* tmp, i
 
 #define UINT3(a, b, c) (size_t)((a << 16) + (b << 8) + c);
 
-static int read_GRIB(reader* r)
+static int read_GRIB(reader* r, int no_alloc)
 {
     unsigned char* tmp  = NULL;
     size_t length       = 0;
@@ -167,7 +180,7 @@ static int read_GRIB(reader* r)
     if (!tmp)
         return GRIB_OUT_OF_MEMORY;
     buf           = grib_new_buffer(c, tmp, buflen);
-    buf->property = GRIB_MY_BUFFER;
+    buf->property = CODES_MY_BUFFER;
 
     tmp[i++] = 'G';
     tmp[i++] = 'R';
@@ -408,11 +421,10 @@ static int read_GRIB(reader* r)
             r->seek_from_start(r->read_data, r->offset + 4);
             grib_buffer_delete(c, buf);
             return GRIB_UNSUPPORTED_EDITION;
-            break;
     }
 
     /* Assert(i <= buf->length); */
-    err = read_the_rest(r, length, tmp, i, 1);
+    err = read_the_rest(r, length, tmp, i, /*check7777=*/1, no_alloc);
     if (err)
         r->seek_from_start(r->read_data, r->offset + 4);
 
@@ -421,7 +433,7 @@ static int read_GRIB(reader* r)
     return err;
 }
 
-static int read_PSEUDO(reader* r, const char* type)
+static int read_PSEUDO(reader* r, const char* type, int no_alloc)
 {
     unsigned char tmp[32]; /* Should be enough */
     size_t sec1len = 0;
@@ -467,10 +479,9 @@ static int read_PSEUDO(reader* r, const char* type)
     /* fprintf(stderr,"%s sec4len=%d i=%d l=%d\n",type,sec4len,i,4+sec1len+sec4len+4); */
 
     Assert(i <= sizeof(tmp));
-    return read_the_rest(r, 4 + sec1len + sec4len + 4, tmp, i, 1);
+    return read_the_rest(r, 4 + sec1len + sec4len + 4, tmp, i, /*check7777=*/1, no_alloc);
 }
 
-
 static int read_HDF5_offset(reader* r, int length, unsigned long* v, unsigned char* tmp, int* i)
 {
     unsigned char buf[8];
@@ -655,7 +666,7 @@ static int read_HDF5(reader* r)
     }
 
     Assert(i <= sizeof(tmp));
-    return read_the_rest(r, end_of_file_address, tmp, i, 0);
+    return read_the_rest(r, end_of_file_address, tmp, i, 0, 0);
 }
 
 static int read_WRAP(reader* r)
@@ -677,7 +688,6 @@ static int read_WRAP(reader* r)
     tmp[i++] = 'P';
 
     if ((r->read(r->read_data, buf, 8, &err) != 8) || err) {
-        printf("error\n");
         return err;
     }
 
@@ -688,10 +698,10 @@ static int read_WRAP(reader* r)
     }
 
     Assert(i <= sizeof(tmp));
-    return read_the_rest(r, length, tmp, i, 1);
+    return read_the_rest(r, length, tmp, i, 1, 0);
 }
 
-static int read_BUFR(reader* r)
+static int read_BUFR(reader* r, int no_alloc)
 {
     /* unsigned char tmp[65536];*/ /* Should be enough */
     size_t length      = 0;
@@ -709,7 +719,7 @@ static int read_BUFR(reader* r)
     if (!tmp)
         return GRIB_OUT_OF_MEMORY;
     buf           = grib_new_buffer(c, tmp, buflen);
-    buf->property = GRIB_MY_BUFFER;
+    buf->property = CODES_MY_BUFFER;
     r->offset     = r->tell(r->read_data) - 4;
 
     tmp[i++] = 'B';
@@ -809,6 +819,9 @@ static int read_BUFR(reader* r)
             GROW_BUF_IF_REQUIRED(sec1len + sec2len + sec3len + 4 + 3);
 
             /* Read section 3 */
+            if (sec3len < 5) {
+                return GRIB_INVALID_MESSAGE; // ECC-1778
+            }
             if ((r->read(r->read_data, tmp + i, sec3len - 3, &err) != sec3len - 3) || err)
                 return err;
             i += sec3len - 3;
@@ -837,7 +850,7 @@ static int read_BUFR(reader* r)
     }
 
     /* Assert(i <= sizeof(tmp)); */
-    err = read_the_rest(r, length, tmp, i, 1);
+    err = read_the_rest(r, length, tmp, i, /*check7777=*/1, no_alloc);
     if (err)
         r->seek_from_start(r->read_data, r->offset + 4);
 
@@ -846,7 +859,7 @@ static int read_BUFR(reader* r)
     return err;
 }
 
-static int ecc_read_any(reader* r, int grib_ok, int bufr_ok, int hdf5_ok, int wrap_ok)
+static int ecc_read_any(reader* r, int no_alloc, int grib_ok, int bufr_ok, int hdf5_ok, int wrap_ok)
 {
     unsigned char c;
     int err             = 0;
@@ -859,14 +872,14 @@ static int ecc_read_any(reader* r, int grib_ok, int bufr_ok, int hdf5_ok, int wr
         switch (magic & 0xffffffff) {
             case GRIB:
                 if (grib_ok) {
-                    err = read_GRIB(r);
+                    err = read_GRIB(r, no_alloc);
                     return err == GRIB_END_OF_FILE ? GRIB_PREMATURE_END_OF_FILE : err; /* Premature EOF */
                 }
                 break;
 
             case BUFR:
                 if (bufr_ok) {
-                    err = read_BUFR(r);
+                    err = read_BUFR(r, no_alloc);
                     return err == GRIB_END_OF_FILE ? GRIB_PREMATURE_END_OF_FILE : err; /* Premature EOF */
                 }
                 break;
@@ -887,19 +900,19 @@ static int ecc_read_any(reader* r, int grib_ok, int bufr_ok, int hdf5_ok, int wr
 
             case BUDG:
                 if (grib_ok) {
-                    err = read_PSEUDO(r, "BUDG");
+                    err = read_PSEUDO(r, "BUDG", no_alloc);
                     return err == GRIB_END_OF_FILE ? GRIB_PREMATURE_END_OF_FILE : err; /* Premature EOF */
                 }
                 break;
             case DIAG:
                 if (grib_ok) {
-                    err = read_PSEUDO(r, "DIAG");
+                    err = read_PSEUDO(r, "DIAG", no_alloc);
                     return err == GRIB_END_OF_FILE ? GRIB_PREMATURE_END_OF_FILE : err; /* Premature EOF */
                 }
                 break;
             case TIDE:
                 if (grib_ok) {
-                    err = read_PSEUDO(r, "TIDE");
+                    err = read_PSEUDO(r, "TIDE", no_alloc);
                     return err == GRIB_END_OF_FILE ? GRIB_PREMATURE_END_OF_FILE : err; /* Premature EOF */
                 }
                 break;
@@ -909,7 +922,7 @@ static int ecc_read_any(reader* r, int grib_ok, int bufr_ok, int hdf5_ok, int wr
     return err;
 }
 
-static int read_any(reader* r, int grib_ok, int bufr_ok, int hdf5_ok, int wrap_ok)
+static int read_any(reader* r, int no_alloc, int grib_ok, int bufr_ok, int hdf5_ok, int wrap_ok)
 {
     int result = 0;
 
@@ -918,11 +931,11 @@ static int read_any(reader* r, int grib_ok, int bufr_ok, int hdf5_ok, int wrap_o
      * so each thread gets its own message. Otherwise if threads are passed
      * different files, then the lock is not needed
      */
-    GRIB_MUTEX_INIT_ONCE(&once, &init);
+    GRIB_MUTEX_INIT_ONCE(&once, &init_mutex);
     GRIB_MUTEX_LOCK(&mutex1);
 #endif
 
-    result = ecc_read_any(r, grib_ok, bufr_ok, hdf5_ok, wrap_ok);
+    result = ecc_read_any(r, no_alloc, grib_ok, bufr_ok, hdf5_ok, wrap_ok);
 
 #ifndef ECCODES_EACH_THREAD_OWN_FILE
     GRIB_MUTEX_UNLOCK(&mutex1);
@@ -938,7 +951,7 @@ static int read_any_gts(reader* r)
     unsigned long magic     = 0;
     unsigned long start     = 0x010d0d0a; /* SOH CR CR LF */
     unsigned long theEnd    = 0x0d0d0a03; /* CR CR LF ETX */
-    unsigned char tmp[1024] = {0,}; /* See ECC-735 */
+    unsigned char tmp[16384] = {0,}; /* See ECC-735 */
     size_t message_size = 0;
     size_t already_read = 0;
     int i               = 0;
@@ -996,7 +1009,7 @@ static int read_any_taf(reader* r)
     int err                 = 0;
     unsigned char* buffer   = NULL;
     unsigned long magic     = 0;
-    unsigned long start     = 0x54414620;
+    unsigned long start     = 0x54414620; // 4 chars: TAF plus a space
     unsigned char tmp[1000] = {0,}; /* Should be enough */
     size_t message_size = 0;
     size_t already_read = 0;
@@ -1008,10 +1021,10 @@ static int read_any_taf(reader* r)
         magic &= 0xffffffff;
 
         if (magic == start) {
-            tmp[i++] = 0x54;
-            tmp[i++] = 0x41;
-            tmp[i++] = 0x46;
-            tmp[i++] = 0x20;
+            tmp[i++] = 0x54; //T
+            tmp[i++] = 0x41; //A
+            tmp[i++] = 0x46; //F
+            tmp[i++] = 0x20; //space
 
             r->offset = r->tell(r->read_data) - 4;
 
@@ -1044,7 +1057,7 @@ static int read_any_metar(reader* r)
     int err               = 0;
     unsigned char* buffer = NULL;
     unsigned long magic   = 0;
-    unsigned long start   = 0x4d455441;
+    unsigned long start   = 0x4d455441; // 4 chars: META
     unsigned char tmp[32] = {0,}; /* Should be enough */
     size_t message_size = 0;
     size_t already_read = 0;
@@ -1059,10 +1072,10 @@ static int read_any_metar(reader* r)
             if (r->read(r->read_data, &c, 1, &err) != 1 || err != 0)
                 break;
             if (c == 'R') {
-                tmp[i++] = 0x4d;
-                tmp[i++] = 0x45;
-                tmp[i++] = 0x54;
-                tmp[i++] = 0x41;
+                tmp[i++] = 0x4d; // M
+                tmp[i++] = 0x45; // E
+                tmp[i++] = 0x54; // T
+                tmp[i++] = 0x41; // A
                 tmp[i++] = 'R';
 
                 r->offset = r->tell(r->read_data) - 4;
@@ -1152,7 +1165,8 @@ static void* user_provider_buffer(void* data, size_t* length, int* err)
     return u->user_buffer;
 }
 
-static int ecc_wmo_read_any_from_file(FILE* f, void* buffer, size_t* len, int grib_ok, int bufr_ok, int hdf5_ok, int wrap_ok)
+static int ecc_wmo_read_any_from_file(FILE* f, void* buffer, size_t* len, off_t* offset,
+                                      int no_alloc, int grib_ok, int bufr_ok, int hdf5_ok, int wrap_ok)
 {
     int err;
     user_buffer_t u;
@@ -1172,25 +1186,60 @@ static int ecc_wmo_read_any_from_file(FILE* f, void* buffer, size_t* len, int gr
     r.offset          = 0;
     r.message_size    = 0;
 
-    err  = read_any(&r, grib_ok, bufr_ok, hdf5_ok, wrap_ok);
+    err  = read_any(&r, no_alloc, grib_ok, bufr_ok, hdf5_ok, wrap_ok);
     *len = r.message_size;
+    *offset = r.offset;
 
     return err;
 }
 
 int wmo_read_any_from_file(FILE* f, void* buffer, size_t* len)
 {
-    return ecc_wmo_read_any_from_file(f, buffer, len, 1, 1, 1, 1);
+    off_t offset = 0;
+    return ecc_wmo_read_any_from_file(f, buffer, len, &offset, /*no_alloc=*/0, 1, 1, 1, 1);
 }
-
 int wmo_read_grib_from_file(FILE* f, void* buffer, size_t* len)
 {
-    return ecc_wmo_read_any_from_file(f, buffer, len, 1, 0, 0, 0);
+    off_t offset = 0;
+    return ecc_wmo_read_any_from_file(f, buffer, len, &offset, /*no_alloc=*/0, 1, 0, 0, 0);
 }
-
 int wmo_read_bufr_from_file(FILE* f, void* buffer, size_t* len)
 {
-    return ecc_wmo_read_any_from_file(f, buffer, len, 0, 1, 0, 0);
+    off_t offset = 0;
+    return ecc_wmo_read_any_from_file(f, buffer, len, &offset, /*no_alloc=*/0, 0, 1, 0, 0);
+}
+
+// Fast versions
+int wmo_read_any_from_file_fast(FILE* f, size_t* msg_len, off_t* msg_offset)
+{
+    unsigned char buffer[64] = {0,};
+    *msg_len = sizeof(buffer);
+    return ecc_wmo_read_any_from_file(f, buffer, msg_len, msg_offset, /*no_alloc=*/1, 1, 1, 1, 1);
+}
+int wmo_read_grib_from_file_fast(FILE* f, size_t* msg_len, off_t* msg_offset)
+{
+    unsigned char buffer[64] = {0,};
+    *msg_len = sizeof(buffer);
+    return ecc_wmo_read_any_from_file(f, buffer, msg_len, msg_offset, /*no_alloc=*/1, 1, 0, 0, 0);
+}
+int wmo_read_bufr_from_file_fast(FILE* f, size_t* msg_len, off_t* msg_offset)
+{
+    unsigned char buffer[64] = {0,};
+    *msg_len = sizeof(buffer);
+    return ecc_wmo_read_any_from_file(f, buffer, msg_len, msg_offset, /*no_alloc=*/1, 0, 1, 0, 0);
+}
+int wmo_read_gts_from_file_fast(FILE* f, size_t* msg_len, off_t* msg_offset)
+{
+    //TODO(masn): Needs proper implementation; no malloc
+    //unsigned char buffer[1024] = {0,};
+    void* mesg   = NULL;
+    int err      = GRIB_SUCCESS;
+    grib_context* c = grib_context_get_default();
+
+    *msg_len = 1024; // sizeof(buffer)
+    mesg = wmo_read_gts_from_file_malloc(f, 0, msg_len, msg_offset, &err);
+    grib_context_free(c, mesg);
+    return err;
 }
 
 int wmo_read_gts_from_file(FILE* f, void* buffer, size_t* len)
@@ -1219,57 +1268,49 @@ int wmo_read_gts_from_file(FILE* f, void* buffer, size_t* len)
     return err;
 }
 
-int wmo_read_taf_from_file(FILE* f, void* buffer, size_t* len)
-{
-    int err;
-    user_buffer_t u;
-    reader r;
+// int wmo_read_taf_from_file(FILE* f, void* buffer, size_t* len)
+// {
+//     int err;
+//     user_buffer_t u;
+//     reader r;
+//     u.user_buffer = buffer;
+//     u.buffer_size = *len;
+//     r.read_data       = f;
+//     r.read            = &stdio_read;
+//     r.alloc_data      = &u;
+//     r.alloc           = &user_provider_buffer;
+//     r.headers_only    = 0;
+//     r.seek            = &stdio_seek;
+//     r.seek_from_start = &stdio_seek_from_start;
+//     r.tell            = &stdio_tell;
+//     r.offset          = 0;
+//     r.message_size    = 0;
+//     err  = read_any_taf(&r);
+//     *len = r.message_size;
+//     return err;
+// }
 
-    u.user_buffer = buffer;
-    u.buffer_size = *len;
-
-    r.read_data       = f;
-    r.read            = &stdio_read;
-    r.alloc_data      = &u;
-    r.alloc           = &user_provider_buffer;
-    r.headers_only    = 0;
-    r.seek            = &stdio_seek;
-    r.seek_from_start = &stdio_seek_from_start;
-    r.tell            = &stdio_tell;
-    r.offset          = 0;
-    r.message_size    = 0;
-
-    err  = read_any_taf(&r);
-    *len = r.message_size;
-
-    return err;
-}
-
-int wmo_read_metar_from_file(FILE* f, void* buffer, size_t* len)
-{
-    int err;
-    user_buffer_t u;
-    reader r;
-
-    u.user_buffer = buffer;
-    u.buffer_size = *len;
-
-    r.read_data       = f;
-    r.read            = &stdio_read;
-    r.alloc_data      = &u;
-    r.alloc           = &user_provider_buffer;
-    r.headers_only    = 0;
-    r.seek            = &stdio_seek;
-    r.seek_from_start = &stdio_seek_from_start;
-    r.tell            = &stdio_tell;
-    r.offset          = 0;
-    r.message_size    = 0;
-
-    err  = read_any_metar(&r);
-    *len = r.message_size;
-
-    return err;
-}
+// int wmo_read_metar_from_file(FILE* f, void* buffer, size_t* len)
+// {
+//     int err;
+//     user_buffer_t u;
+//     reader r;
+//     u.user_buffer = buffer;
+//     u.buffer_size = *len;
+//     r.read_data       = f;
+//     r.read            = &stdio_read;
+//     r.alloc_data      = &u;
+//     r.alloc           = &user_provider_buffer;
+//     r.headers_only    = 0;
+//     r.seek            = &stdio_seek;
+//     r.seek_from_start = &stdio_seek_from_start;
+//     r.tell            = &stdio_tell;
+//     r.offset          = 0;
+//     r.message_size    = 0;
+//     err  = read_any_metar(&r);
+//     *len = r.message_size;
+//     return err;
+// }
 
 /*================== */
 
@@ -1297,7 +1338,7 @@ static size_t stream_read(void* data, void* buffer, size_t len, int* err)
     if (len > LONG_MAX) {
         /* size_t cannot be coded into long */
         *err = GRIB_INTERNAL_ERROR;
-        return -1;
+        return 0;
     }
 
     n = s->stream_proc(s->stream_data, buffer, len);
@@ -1346,7 +1387,7 @@ int wmo_read_any_from_stream(void* stream_data, long (*stream_proc)(void*, void*
     r.alloc           = &user_provider_buffer;
     r.headers_only    = 0;
 
-    err  = read_any(&r, 1, 1, 1, 1);
+    err  = read_any(&r, /*no_alloc=*/0, 1, 1, 1, 1);
     *len = r.message_size;
 
     return err;
@@ -1375,7 +1416,7 @@ void* wmo_read_any_from_stream_malloc(void* stream_data, long (*stream_proc)(voi
     r.alloc           = &allocate_buffer;
     r.headers_only    = 0;
 
-    *err  = read_any(&r, 1, 1, 1, 1);
+    *err  = read_any(&r, /*no_alloc=*/0, 1, 1, 1, 1);
     *size = r.message_size;
 
     return u.buffer;
@@ -1482,7 +1523,7 @@ static void* ecc_wmo_read_any_from_file_malloc(FILE* f, int* err, size_t* size,
     r.headers_only    = headers_only;
     r.offset          = 0;
 
-    *err = read_any(&r, grib_ok, bufr_ok, hdf5_ok, wrap_ok);
+    *err = read_any(&r, /*no_alloc=*/0, grib_ok, bufr_ok, hdf5_ok, wrap_ok);
 
     *size   = r.message_size;
     *offset = r.offset;
@@ -1546,7 +1587,7 @@ int grib_read_any_headers_only_from_file(grib_context* ctx, FILE* f, void* buffe
     r.offset          = 0;
     r.message_size    = 0;
 
-    err = read_any(&r, 1, ECCODES_READS_BUFR, ECCODES_READS_HDF5, ECCODES_READS_WRAP);
+    err = read_any(&r, /*no_alloc=*/0, 1, ECCODES_READS_BUFR, ECCODES_READS_HDF5, ECCODES_READS_WRAP);
 
     *len = r.message_size;
 
@@ -1576,7 +1617,7 @@ int grib_read_any_from_file(grib_context* ctx, FILE* f, void* buffer, size_t* le
 
     offset = ftello(f);
 
-    err = read_any(&r, 1, ECCODES_READS_BUFR, ECCODES_READS_HDF5, ECCODES_READS_WRAP);
+    err = read_any(&r, /*no_alloc=*/0, 1, ECCODES_READS_BUFR, ECCODES_READS_HDF5, ECCODES_READS_WRAP);
 
     if (err == GRIB_BUFFER_TOO_SMALL) {
         if (fseeko(f, offset, SEEK_SET))
@@ -1647,7 +1688,7 @@ int grib_read_any_from_memory_alloc(grib_context* ctx, unsigned char** data, siz
     r.offset          = 0;
     r.message_size    = 0;
 
-    err     = read_any(&r, 1, ECCODES_READS_BUFR, ECCODES_READS_HDF5, ECCODES_READS_WRAP);
+    err     = read_any(&r, /*no_alloc=*/0, 1, ECCODES_READS_BUFR, ECCODES_READS_HDF5, ECCODES_READS_WRAP);
     *buffer = u.buffer;
     *length = u.length;
 
@@ -1681,9 +1722,8 @@ int grib_read_any_from_memory(grib_context* ctx, unsigned char** data, size_t* d
     r.offset          = 0;
     r.message_size    = 0;
 
-    err  = read_any(&r, 1, ECCODES_READS_BUFR, ECCODES_READS_HDF5, ECCODES_READS_WRAP);
+    err  = read_any(&r, /*no_alloc=*/0, 1, ECCODES_READS_BUFR, ECCODES_READS_HDF5, ECCODES_READS_WRAP);
     *len = r.message_size;
-
     *data_length = m.data_len;
     *data        = m.data;
 
@@ -1706,11 +1746,9 @@ int grib_count_in_file(grib_context* c, FILE* f, int* n)
         }
     }
     else {
-        void* mesg   = NULL;
         size_t size  = 0;
         off_t offset = 0;
-        while ((mesg = wmo_read_any_from_file_malloc(f, 0, &size, &offset, &err)) != NULL && err == GRIB_SUCCESS) {
-            grib_context_free(c, mesg);
+        while ((err = wmo_read_any_from_file_fast(f, &size, &offset)) == GRIB_SUCCESS) {
             (*n)++;
         }
     }
@@ -1737,15 +1775,15 @@ int grib_count_in_filename(grib_context* c, const char* filename, int* n)
     return err;
 }
 
-typedef void* (*decoder_proc)(FILE* f, int headers_only, size_t* size, off_t* offset, int* err);
+typedef int (*decoder_proc)(FILE* f, size_t* size, off_t* offset);
 
 static decoder_proc get_reader_for_product(ProductKind product)
 {
     decoder_proc decoder = NULL;
-    if      (product == PRODUCT_GRIB) decoder = &wmo_read_grib_from_file_malloc;
-    else if (product == PRODUCT_BUFR) decoder = &wmo_read_bufr_from_file_malloc;
-    else if (product == PRODUCT_GTS)  decoder = &wmo_read_gts_from_file_malloc;
-    else if (product == PRODUCT_ANY)  decoder = &wmo_read_any_from_file_malloc;
+    if      (product == PRODUCT_GRIB) decoder = &wmo_read_grib_from_file_fast;
+    else if (product == PRODUCT_BUFR) decoder = &wmo_read_bufr_from_file_fast;
+    else if (product == PRODUCT_GTS)  decoder = &wmo_read_gts_from_file_fast;
+    else if (product == PRODUCT_ANY)  decoder = &wmo_read_any_from_file_fast;
     return decoder;
 }
 
@@ -1759,20 +1797,18 @@ static int count_product_in_file(grib_context* c, FILE* f, ProductKind product,
     decoder = get_reader_for_product(product);
 
     if (!decoder) {
-        grib_context_log(c, GRIB_LOG_ERROR, "count_product_in_file: not supported for given product");
+        grib_context_log(c, GRIB_LOG_ERROR, "%s: Not supported for given product", __func__);
         return GRIB_INVALID_ARGUMENT;
     }
 
     if (c->multi_support_on && product == PRODUCT_GRIB) {
-        grib_context_log(c, GRIB_LOG_ERROR, "count_product_in_file: Multi-field GRIBs not supported");
+        grib_context_log(c, GRIB_LOG_ERROR, "%s: Multi-field GRIBs not supported", __func__);
         err = GRIB_NOT_IMPLEMENTED;
     }
     else {
-        void* mesg   = NULL;
         size_t size  = 0;
         off_t offset = 0;
-        while ((mesg = decoder(f, 0, &size, &offset, &err)) != NULL && err == GRIB_SUCCESS) {
-            grib_context_free(c, mesg);
+        while ((err = decoder(f, &size, &offset)) == GRIB_SUCCESS) {
             (*count)++;
         }
         rewind(f);
@@ -1781,10 +1817,13 @@ static int count_product_in_file(grib_context* c, FILE* f, ProductKind product,
     return err == GRIB_END_OF_FILE ? 0 : err;
 }
 
-int codes_extract_offsets_malloc(grib_context* c, const char* filename, ProductKind product, off_t** offsets, int* length, int strict_mode)
+static int codes_extract_offsets_malloc_internal(
+    grib_context* c, const char* filename, ProductKind product,
+    off_t** offsets, size_t** sizes,
+    int* number_of_elements,
+    int strict_mode)
 {
     int err      = 0;
-    void* mesg   = NULL;
     size_t size  = 0;
     off_t offset = 0;
     int num_messages = 0, i = 0;
@@ -1793,26 +1832,31 @@ int codes_extract_offsets_malloc(grib_context* c, const char* filename, ProductK
 
     decoder = get_reader_for_product(product);
     if (!decoder) {
-        grib_context_log(c, GRIB_LOG_ERROR, "codes_extract_offsets_malloc: not supported for given product");
+        grib_context_log(c, GRIB_LOG_ERROR, "%s: Not supported for given product", __func__);
         return GRIB_INVALID_ARGUMENT;
     }
     if (!c) c = grib_context_get_default();
+
+    if (path_is_directory(filename)) {
+        grib_context_log(c, GRIB_LOG_ERROR, "%s: \"%s\" is a directory", __func__, filename);
+        return GRIB_IO_PROBLEM;
+    }
     f = fopen(filename, "rb");
     if (!f) {
-        grib_context_log(c, GRIB_LOG_ERROR, "codes_extract_offsets_malloc: Unable to read file \"%s\"", filename);
+        grib_context_log(c, GRIB_LOG_ERROR, "%s: Unable to read file \"%s\"", __func__, filename);
         perror(filename);
         return GRIB_IO_PROBLEM;
     }
 
     err = count_product_in_file(c, f, product, &num_messages);
     if (err) {
-        grib_context_log(c, GRIB_LOG_ERROR, "codes_extract_offsets_malloc: Unable to count messages");
+        grib_context_log(c, GRIB_LOG_ERROR, "%s: Unable to count messages (%s)", __func__, grib_get_error_message(err));
         fclose(f);
         return err;
     }
-    *length = num_messages;
+    *number_of_elements = num_messages;
     if (num_messages == 0) {
-        grib_context_log(c, GRIB_LOG_ERROR, "codes_extract_offsets_malloc: No messages in file");
+        grib_context_log(c, GRIB_LOG_ERROR, "%s: No messages in file", __func__);
         fclose(f);
         return GRIB_INVALID_MESSAGE;
     }
@@ -1821,32 +1865,30 @@ int codes_extract_offsets_malloc(grib_context* c, const char* filename, ProductK
         fclose(f);
         return GRIB_OUT_OF_MEMORY;
     }
+    if (sizes) {
+        *sizes = (size_t*)calloc(num_messages, sizeof(size_t));
+        if (!*sizes) {
+            fclose(f);
+            return GRIB_OUT_OF_MEMORY;
+        }
+    }
 
     i = 0;
     while (err != GRIB_END_OF_FILE) {
         if (i >= num_messages)
             break;
 
-        mesg = decoder(f, 0, &size, &offset, &err);
-        if (mesg != NULL && err == 0) {
+        err = decoder(f, &size, &offset);
+        if (!err) {
             (*offsets)[i] = offset;
-            grib_context_free(c, mesg);
-        }
-        if (mesg && err) {
-            if (strict_mode) {
-                grib_context_free(c, mesg);
-                fclose(f);
-                return GRIB_DECODING_ERROR;
+            if (sizes) {
+                (*sizes)[i] = size;
             }
         }
-        if (!mesg) {
-            if (err != GRIB_END_OF_FILE && err != GRIB_PREMATURE_END_OF_FILE) {
-                /* An error occurred */
-                grib_context_log(c, GRIB_LOG_ERROR, "codes_extract_offsets_malloc: Unable to read message");
-                if (strict_mode) {
-                    fclose(f);
-                    return GRIB_DECODING_ERROR;
-                }
+        else {
+            if (strict_mode && (err != GRIB_END_OF_FILE && err != GRIB_PREMATURE_END_OF_FILE)) {
+                fclose(f);
+                return GRIB_DECODING_ERROR;
             }
         }
         ++i;
@@ -1855,3 +1897,20 @@ int codes_extract_offsets_malloc(grib_context* c, const char* filename, ProductK
     fclose(f);
     return err;
 }
+
+// The lagacy version only did the offsets
+int codes_extract_offsets_malloc(
+    grib_context* c, const char* filename, ProductKind product,
+    off_t** offsets, int* number_of_elements, int strict_mode)
+{
+    // Call without doing the message sizes
+    return codes_extract_offsets_malloc_internal(c, filename, product, offsets, NULL, number_of_elements, strict_mode);
+}
+
+// New function does both message offsets and sizes
+int codes_extract_offsets_sizes_malloc(
+    grib_context* c, const char* filename, ProductKind product,
+    off_t** offsets, size_t** sizes, int* number_of_elements, int strict_mode)
+{
+    return codes_extract_offsets_malloc_internal(c, filename, product, offsets, sizes, number_of_elements, strict_mode);
+}
