@@ -8,14 +8,15 @@
 # virtue of its status as an intergovernmental organisation nor does it submit to any jurisdiction.
 #
 
-set -x
-. ./include.sh
-
-cd ${data_dir}/bufr
+. ./include.ctest.sh
 
 # Define a common label for all the tmp files
 label="bufr_filter_misc_test"
 
+tempErr=temp.$label.err
+
+cd ${data_dir}/bufr
+
 # Create log file
 fLog=${label}".log"
 rm -f $fLog
@@ -118,7 +119,7 @@ for statid  in 1 3 7 ; do
 done
 
 #-----------------------------------------------------------
-# Test:  print attributes
+# Test: print attributes
 #-----------------------------------------------------------
 cat > $fRules <<EOF
 set unpack=1;
@@ -158,7 +159,7 @@ diff ${f}.ref ${f}.log
 rm -f ${f}.ref ${f}.log
 
 #-----------------------------------------------------------
-# Test:  access element by rank
+# Test: access element by rank
 #-----------------------------------------------------------
 cat > $fRules <<EOF
 set unpack=1;
@@ -191,7 +192,7 @@ diff ${f}.ref ${f}.log
 rm -f ${f}.ref ${f}.log
 
 #-----------------------------------------------------------
-# Test:  access marker operators
+# Test: access marker operators
 #-----------------------------------------------------------
 cat > $fRules <<EOF
 set unpack=1;
@@ -265,7 +266,7 @@ diff ${f}.ref ${f}.log
 rm -f ${f}.ref ${f}.log
 
 #-----------------------------------------------------------
-# Test:  access marker operators 2
+# Test: access marker operators 2
 #-----------------------------------------------------------
 cat > $fRules <<EOF
 set unpack=1;
@@ -318,7 +319,7 @@ diff ${f}.ref ${f}.log
 rm -f ${f}.ref ${f}.log
 
 #-----------------------------------------------------------
-# Test:  access by condition
+# Test: access by condition
 #-----------------------------------------------------------
 cat > $fRules <<EOF
 set unpack=1;
@@ -339,7 +340,7 @@ diff $REFERENCE_FILE ${f}.log
 rm -f ${f}.log
 
 #-----------------------------------------------------------
-# Test:  access by condition 2
+# Test: access by condition 2
 #-----------------------------------------------------------
 cat > $fRules <<EOF
 set unpack=1;
@@ -419,7 +420,7 @@ rm -f ${f}.ref ${f}.log
 #rm -f $testScript $testScript1
 
 #-----------------------------------------------------------
-# Test:  get string
+# Test: get string
 #-----------------------------------------------------------
 cat > $fRules <<EOF
 set unpack=1;
@@ -452,7 +453,7 @@ f="$ECCODES_SAMPLES_PATH/BUFR4.tmpl"
 ${tools_dir}/codes_bufr_filter $fRules $f
 
 #-----------------------------------------------------------
-# Test:  get string whose value is MISSING (ECC-650)
+# Test: get string whose value is MISSING (ECC-650)
 #-----------------------------------------------------------
 cat > $fRules <<EOF
  set unpack=1;
@@ -467,7 +468,7 @@ diff ${f}.ref ${f}.log
 rm -f ${f}.ref ${f}.log
 
 #-----------------------------------------------------------
-# Test:  get string array and stringValues
+# Test: get string array and stringValues
 #-----------------------------------------------------------
 cat > $fRules <<EOF
  set unpack=1;
@@ -516,54 +517,6 @@ diff ${f}.ref ${f}.log
 
 rm -f ${f}.ref ${f}.log
 
-#-----------------------------------------------------------
-# Test: with nonexistent keys.
-#-----------------------------------------------------------
-cat > $fRules <<EOF
- set center="98";   #Here centre is misspelled
-EOF
-
-# Invoke without -f i.e. should fail if error encountered
-set +e
-
-f="syno_1.bufr"
-echo "Test: nonexistent keys" >> $fLog
-echo "file: $f" >> $fLog
-${tools_dir}/codes_bufr_filter $fRules $f 2>> $fLog 1>> $fLog
-if [ $? -eq 0 ]; then
-   echo "bufr_filter should have failed if key not found" >&2
-   exit 1
-fi
-set -e
-
-# Now repeat with -f option (do not exit on error)
-${tools_dir}/codes_bufr_filter -f $fRules $f 2>>$fLog 1>>$fLog
-
-
-#-----------------------------------------------------------
-# Test: with not allowed key values
-#-----------------------------------------------------------
-cat > $fRules <<EOF
- set centre=1024;  #1024 is out of range (it is 8-bit only)
-EOF
-
-# Invoke without -f i.e. should fail if error encountered
-set +e
-
-f="syno_1.bufr"
-echo "Test: not allowed key values" >> $fLog
-echo "file: $f" >> $fLog
-${tools_dir}/codes_bufr_filter $fRules $f 2>> $fLog 1>> $fLog
-if [ $? -eq 0 ]; then
-   echo "bufr_filter should have failed if key value is not allowed" >&2
-   exit 1
-fi
-set -e
-
-# Now repeat with -f option (do not exit on error)
-${tools_dir}/codes_bufr_filter -f $fRules $f 2>>$fLog 1>>$fLog
-
-
 #----------------------------------------------------
 # Test: format specifier for integer keys
 #----------------------------------------------------
@@ -694,37 +647,10 @@ EOF
 
 ${tools_dir}/codes_bufr_filter -o ${fout} $fRules $f 2>> $fLog 1>> $fLog
 ${tools_dir}/bufr_compare $fout ${fout}.ref #2>> $fLog 1>> $fLog
+rm -f $fout
 
 #-----------------------------------------------------------
-# ECC-147
-#-----------------------------------------------------------
-cat > $fRules <<EOF
- set unpack=1;
- set relativeHumidity=27;
- set horizontalVisibility=1500;
- set pack=1;
- write;
-EOF
-
-f="syno_1.bufr"
-${tools_dir}/codes_bufr_filter -o ${f}.out $fRules $f
-# This part of the test is meant to fail
-set +e
-${tools_dir}/bufr_compare ${f}.out $f
-status=$?
-set -e
-if [ $status -eq 0 ]; then
-  # compare should have failed and returned a non-zero exit code
-  exit 1
-fi
-# Now blacklist the failing keys and it should pass
-${tools_dir}/bufr_compare -b relativeHumidity,horizontalVisibility ${f}.out $f
-
-rm -f ${f}.out 
-
-rm -f $fRules ${fout} $fLog
-#-----------------------------------------------------------
-# Test:  access subsets by condition 
+# Test: access subsets by condition 
 #-----------------------------------------------------------
 cat > $fRules <<EOF
  set unpack=1;
@@ -762,7 +688,7 @@ diff ${f}.ref ${f}.log
 rm -f ${f}.ref ${f}.log
 
 #-----------------------------------------------------------
-# Test:  access subsets and attribute by condition 
+# Test: access subsets and attribute by condition 
 #-----------------------------------------------------------
 cat > $fRules <<EOF
  set unpack=1;
@@ -784,7 +710,7 @@ diff ${f}.ref ${f}.log
 rm -f ${f}.ref ${f}.log
 rm -f $fLog $fRules
 #-----------------------------------------------------------
-# Test:  set key by rank                           
+# Test: set key by rank                           
 #-----------------------------------------------------------
 cat > $fRules <<EOF
  set unpack=1;
@@ -807,7 +733,7 @@ diff ${f}.ref ${f}.log
 rm -f ${f}.ref ${f}.log
 rm -f $fLog $fRules
 #-----------------------------------------------------------
-# Test:  initialise with given values of delayed replications
+# Test: initialise with given values of delayed replications
 #-----------------------------------------------------------
 cat > $fRules <<EOF
  set localTablesVersionNumber=0;
@@ -857,7 +783,7 @@ ${tools_dir}/bufr_compare ${fOut} ${fOut}.ref
 rm -f ${fOut}.log
 rm -f $fLog $fRules ${fOut} ${fOut}.log.ref
 #-----------------------------------------------------------
-# Test:  add section 2
+# Test: add section 2
 #-----------------------------------------------------------
 cat > $fRules <<EOF
  set bufrHeaderCentre=98;
@@ -890,7 +816,7 @@ rm -f ${fOut}.log
 rm -f $fLog $fRules ${fOut} ${fOut}.log.ref
 
 #-----------------------------------------------------------
-# Test:  associatedField
+# Test: associatedField
 #-----------------------------------------------------------
 cat > $fRules <<EOF
 set unpack=1;
@@ -916,7 +842,7 @@ rm -f ${f}.log ${f}.log.ref
 rm -f $fLog $fRules 
 
 #-----------------------------------------------------------
-# Test:  firstOrderStatistics
+# Test: firstOrderStatistics
 #-----------------------------------------------------------
 cat > $fRules <<EOF
 set unpack=1;
@@ -951,7 +877,7 @@ rm -f ${f}.log ${f}.log.ref
 rm -f $fLog $fRules 
 
 #-----------------------------------------------------------
-# Test:  delayed replication compressed data
+# Test: delayed replication compressed data
 #-----------------------------------------------------------
 cat > $fRules <<EOF
  set localTablesVersionNumber=1;
@@ -1001,7 +927,7 @@ rm -f ${f}.log ${f}.log.ref
 rm -f $fLog $fOut $fRules 
 
 #-----------------------------------------------------------
-# Test:  create new BUFR with bitmap
+# Test: create new BUFR with bitmap
 #-----------------------------------------------------------
 cat > $fRules <<EOF
 set compressedData=1;
@@ -1049,7 +975,7 @@ rm -f ${f}.log ${f}.log.ref
 rm -f $fLog $fOut $fRules 
 
 #-----------------------------------------------------------
-# Test:  several percentConfidence for same element
+# Test: several percentConfidence for same element
 #-----------------------------------------------------------
 cat > $fRules <<EOF
 set unpack=1;
@@ -1261,76 +1187,6 @@ diff ${f}.log.ref ${f}.log
 
 rm -f ${f}.log ${f}.log.ref ${f}.out $fLog $fRules
 
-#-----------------------------------------------------------
-# Test: fix for ECC-389 
-#-----------------------------------------------------------
-cat > $fRules <<EOF
- set numberOfSubsets=2;
- set unexpandedDescriptors={310008};
- set #14#brightnessTemperature={266.53,266.53000000001};
- set pack=1;
- write;
-EOF
-
-f="amsu_55.bufr"
-
-echo "Test: fix for ECC-389" >> $fLog
-echo "file: $f" >> $fLog
-
-${tools_dir}/codes_bufr_filter -o ${f}.out $fRules $f
-
-cat > $fRules <<EOF
-set unpack=1;
-print "[#14#brightnessTemperature]";
-EOF
-
-${tools_dir}/codes_bufr_filter $fRules ${f}.out > ${f}.log
-
-cat > ${f}.log.ref <<EOF
-266.53
-EOF
-
-diff ${f}.log.ref ${f}.log 
-
-rm -f ${f}.log ${f}.log.ref ${f}.out $fLog $fRules
-
-#-----------------------------------------------------------
-# Test: change width using operator 201YYY
-#-----------------------------------------------------------
-HIGH_TEMPERATURE=10000
-cat > $fRules <<EOF
- set unpack=1;
- set airTemperature=$HIGH_TEMPERATURE;
- set pack=1;
- write;
-EOF
-f="bssh_176.bufr"
-
- # This should fail. Out of Range
-set +e
-${tools_dir}/codes_bufr_filter -o ${f}.out $fRules $f
-status=$?
-set -e
-[ $status -ne 0 ]
-
-# Now change the width of airTemperature to allow high value
-cat > $fRules <<EOF
- set unpack=1;
- set edition=4;
- set unexpandedDescriptors={301022,12023,201138,12101,201000,12023};
- set airTemperature=$HIGH_TEMPERATURE;
- set pack=1;
- print "airTemperature=[airTemperature], width=[airTemperature->width]";
- write;
-EOF
-${tools_dir}/codes_bufr_filter -o ${f}.out $fRules $f > ${f}.log
-
-cat > ${f}.log.ref <<EOF
-airTemperature=$HIGH_TEMPERATURE, width=26
-EOF
-diff ${f}.log.ref ${f}.log
-
-rm -f ${f}.log ${f}.log.ref ${f}.out $fLog $fRules
 #-----------------------------------------------------------
 # Test: DateTime
 #-----------------------------------------------------------
@@ -1389,6 +1245,76 @@ userDateTimeStart=2457896.88347 userDateStart=20170523 userTimeStart=91212 20170
 userDateTimeStart=2457896.88347 userDateStart=20170523 userTimeStart=91212 20170523091212
 EOF
 
-diff ${f}.log.ref ${f}.log 
+diff ${f}.log.ref ${f}.log
+rm -f $f.log ${f}.log.ref
 
+
+
+# Decode expandedDescriptors as array of string
+cat > $fRules <<EOF
+ print "[expandedDescriptors:s]";
+EOF
+${tools_dir}/codes_bufr_filter $fRules airc_142.bufr > $fLog
+fRef=temp.$label.ref
+cat > $fRef <<EOF
+001006 002061 004001 004002 004003 004004 004005 005001 
+006001 008004 007002 012001 011001 011002 011031 011032 
+011033 020041 222000 031031 031031 031031 031031 031031 
+031031 031031 031031 031031 031031 031031 031031 031031 
+031031 031031 031031 031031 031031 001031 001032 033007 
+033007 033007 033007 033007 033007 033007 033007 033007 
+033007 033007 033007 033007 033007 033007 033007 033007 
+033007
+EOF
+diff $fRef $fLog
+rm -f $fRef
+
+
+# Decode expandedDescriptors as array of doubles
+cat > $fRules <<EOF
+ print "[expandedDescriptors:d]";
+EOF
+${tools_dir}/codes_bufr_filter $fRules airc_142.bufr
+
+# Various expanded descriptors
+f="$ECCODES_SAMPLES_PATH/BUFR4.tmpl"
+cat > $fRules <<EOF
+  meta expandedScales     expanded_descriptors(elemetsTable,expandedCodes,1);
+  meta expandedReferences expanded_descriptors(elemetsTable,expandedCodes,2);
+  meta expandedWidths     expanded_descriptors(elemetsTable,expandedCodes,3);
+  meta expandedTypes      expanded_descriptors(elemetsTable,expandedCodes,4);
+  print "scales=[expandedScales]";
+  print "refs=[expandedReferences]";
+  print "widths=[expandedWidths]";
+  print "types=[expandedTypes]";
+EOF
+${tools_dir}/codes_bufr_filter $fRules $f > $fLog
+
+# smart table unpack_string
+tempBufr=temp.$label.bufr
+f="$ECCODES_SAMPLES_PATH/BUFR4.tmpl"
+cat > $fRules <<EOF
+  set unexpandedDescriptors = { 1031 };
+  print "[expandedOriginalCodes:s]";
+EOF
+${tools_dir}/codes_bufr_filter -o $tempBufr $fRules $f
+rm -f $tempBufr
+
+# hash_array (sequences) as string and double
+# --------------------------------------------
+cat >$fRules <<EOF
+ set sequences="301012"; print "[sequences]";
+EOF
+result=$(${tools_dir}/codes_bufr_filter $fRules $f)
+[ "$result" = '4004 4005' ]
+
+cat >$fRules <<EOF
+ set sequences=301012.0; print "[sequences]";
+EOF
+result=$(${tools_dir}/codes_bufr_filter $fRules $f)
+[ "$result" = '4004 4005' ]
+
+
+# Clean up
 rm -f ${f}.log ${f}.log.ref ${f}.out $fLog $fRules
+rm -f $tempErr
